#ifndef _CEGOXS_H_INCLUDED
#define _CEGOXS_H_INCLUDED   1

#define NEED_DBIXS_VERSION 93
/* #include <sys/vnode.h> */
#include <sys/stat.h>

#include <DBIXS.h>
#include "dbdimp.h"
#include <dbd_xsh.h>

#endif
